@@ -48,7 +48,7 @@ ldapserver_list_free (ldap_server_t servers)
 
 /* Parse a single LDAP server configuration line.  Returns the server
  * or NULL in case of errors.  The configuration line is assumed to be
- * colon seprated with these fields:
+ * colon separated with these fields:
  *
  * 1. field: Hostname
  * 2. field: Portnumber
@@ -60,11 +60,12 @@ ldapserver_list_free (ldap_server_t servers)
  * Flags are:
  *
  *   starttls  := Use STARTTLS with a default port of 389
- *   ldaptls   := Tunnel LDAP trough a TLS tunnel with default port 636
+ *   ldaptls   := Tunnel LDAP through a TLS tunnel with default port 636
  *   plain     := Switch to plain unsecured LDAP.
  *   (The last of these 3 flags is the effective one)
  *   ntds      := Use Active Directory authentication
  *   areconly  := Use option LDAP_OPT_AREC_EXCLUSIVE
+ *   upload    := Use this server only for upload.
  *
  * FILENAME and LINENO are used for diagnostic purposes only.
  */
@@ -73,9 +74,11 @@ ldapserver_parse_one (const char *line,
 		      const char *filename, unsigned int lineno)
 {
   char *p;
+  const char *s;
   ldap_server_t server;
   int fieldno;
   int fail = 0;
+  int i;
   char **fields = NULL;
 
   server = xtrycalloc (1, sizeof *server);
@@ -143,8 +146,6 @@ ldapserver_parse_one (const char *line,
         case 5:
           {
             char **flags = NULL;
-            int i;
-            const char *s;
 
             flags = strtokenize (p, ",");
             if (!flags)
@@ -182,6 +183,10 @@ ldapserver_parse_one (const char *line,
                   {
                     server->areconly = 1;
                   }
+                else if (!ascii_strcasecmp (s, "upload"))
+                  {
+                    server->upload = 1;
+                  }
                 else
                   {
                     if (filename)
